@@ -758,6 +758,20 @@ Given 'a cookbook recipe with a service resource with an action specified via a
   }.strip
 end
 
+Given 'a cookbook recipe with multiple execute resources where the last uses git' do
+  write_recipe %q{
+    execute "one" do
+      command "ls -al"
+    end
+    execute "two" do
+      command "df -H"
+    end
+    execute "three" do
+      command "git clone https://example.org/bar.git"
+    end
+  }.strip
+end
+
 Given 'a cookbook template that uses all variables passed' do
   write_recipe %q{
     template "/tmp/config.conf" do
@@ -1045,13 +1059,10 @@ Given 'a cookbook with a single recipe that mixes node access types in an interp
   }
 end
 
-Given 'a cookbook with a single recipe that reads multiple node attributes via symbols,strings' do
+Given 'a cookbook with a single recipe that accesses multiple node attributes via symbols' do
   write_recipe %q{
     node[:foo] = 'bar'
-    node[:baz] = 'foo'
-    node[:wham] = 'shazam'
-    node['testing'] = 'bar'
-    node['testing2'] = 'bar2'
+    node[:testing] = 'bar'
   }
 end
 
@@ -1159,12 +1170,12 @@ Given 'a cookbook with a single recipe which accesses node attributes with symbo
     # Here we access the node attributes via a symbol
     foo = node[:foo]
 
-    # String access is in the majority
-    node['foo']
-    node['bar']
-    node['baz']
+    directory "/tmp/foo" do
+      owner "root"
+      group "root"
+      action :create
+    end
 
-    # Second access via a symbol
     bar = node[:bar]
   }
 end
@@ -1203,6 +1214,12 @@ Given /^a cookbook with a single recipe that searches but checks first( \(string
   }
 end
 
+Given 'a cookbook with a single recipe that searches but checks first (ternary) to see if this is server' do
+  write_recipe %Q{
+    required_node = Chef::Config[:solo] ? node : search(:node, query).first
+  }
+end
+
 Given /^a cookbook with a single recipe that searches but checks with a negative first to see if this is server$/ do
   write_recipe %q{
     unless Chef::Config['solo']
@@ -1437,6 +1454,40 @@ Given 'a resource declared with a guard within a loop with multiple block argume
   }
 end
 
+Given 'a resource that declares a guard containing a block' do
+  write_recipe %q{
+    template '/etc/foo' do
+      not_if do
+        s = false
+        node['mylist'].each do |realm|
+          s = node['mylist'][realm].empty?
+          break if s
+        end
+        s
+      end
+      owner 'root'
+      group 'root'
+      mode '0644'
+      source 'foo.erb'
+    end
+  }
+end
+
+
+Given 'a resource declared within a definition' do
+  write_recipe %q{
+    define :toto, {
+    } do
+      [:a, :b].each do |x|
+        package x do
+          not_if { node['foo'] == x }
+          action :install
+        end
+      end
+    end
+  }
+end
+
 Given /^a rule that (declares|does not declare) a version constraint(?: of ([^ ]+)? to ([^ ]+)?)?$/ do |constraint, from, to|
   if from || to
     rule_with_version_constraint(from, to)
@@ -1736,6 +1787,10 @@ When /^I check both cookbooks with the command-line (.*)$/ do |command_line|
   run_lint(cmds)
 end
 
+When 'I check both roles directories' do
+  run_lint ['-R', 'roles1', '-R', 'roles2']
+end
+
 When 'I check the cookbooks, role and environment together' do
   run_lint([
     '-B', 'cookbooks/another_example', '-B', 'cookbooks/example',
@@ -1752,14 +1807,15 @@ When 'I check the environment directory' do
   run_lint ['-E', 'environments']
 end
 
-When 'I check both roles directories' do
-  run_lint ['-R', 'roles1', '-R', 'roles2']
-end
-
 When 'I check the eu environment file only' do
   run_lint ['-E', 'environments/production_eu.rb']
 end
 
+When /^I check the cookbook( without)? excluding the ([^ ]+) directory$/ do |no_exclude, dir|
+  options = no_exclude.nil? ? ['-X', dir] : []
+  run_lint(options + ['cookbooks/example'])
+end
+
 When 'I check the recipe' do
   run_lint(["cookbooks/example/recipes/default.rb"])
 end
@@ -1852,11 +1908,11 @@ Then 'a warning for the custom rule should be displayed' do
 end
 
 Then 'all options should be documented in the man page' do
-  man_page_options.should == usage_options_for_diff
+  man_page_options.must_equal usage_options_for_diff
 end
 
 Then /^an? '([^']+)' error should be displayed$/ do |expected_error|
-  last_error.should include expected_error
+  last_error.must_include expected_error
 end
 
 Then 'the attribute consistency warning 019 should be shown for both of the recipes that use symbols' do
@@ -1873,6 +1929,10 @@ Then /^the bare attribute keys warning 044 should not be displayed against the (
   expect_warning 'FC044', {:expect_warning => false, :line => 2, :file_type => :attributes}
 end
 
+Then 'the execute resource used to run git commands warning 040 should be displayed against the last resource' do
+  expect_warning 'FC040', {:line => 7}
+end
+
 Then /^the LWRP does not notify when updated warning 017 should( not)? be shown against the :([^ ]+) action$/ do |not_shown, action|
   line = action == 'create' ? 1 : 8
   expect_warning('FC017', :file_type => :provider, :expect_warning => ! not_shown, :line => line)
@@ -1914,15 +1974,19 @@ end
 
 Then /^the lint task will be listed( under the different name)?$/ do |diff_name|
   expected_name = diff_name ? 'lint' : 'foodcritic'
-  build_tasks.should include([expected_name, 'Lint Chef cookbooks'])
+  build_tasks.must_include([expected_name, 'Lint Chef cookbooks'])
 end
 
 Then 'no error should have occurred' do
   assert_no_error_occurred
 end
 
-Then /^no warnings will be displayed against the tests$/ do
-  assert_no_test_warnings
+Then /^(no )?warnings will be displayed against the tests$/ do |no_display|
+  if no_display.nil?
+    assert_test_warnings
+  else
+    assert_no_test_warnings
+  end
 end
 
 Then 'the attribute consistency warning 019 should warn on lines 2 and 10 in that order' do
@@ -1982,7 +2046,7 @@ Then /^the build will (succeed|fail) with (?:no )?warnings(.*)$/ do |build_outco
 end
 
 Then 'the check for server warning 003 should not be displayed against the condition' do
-  expect_warning("FC003", :line => 5, :expect_warning => false)
+  expect_warning("FC003", :line => nil, :expect_warning => false)
 end
 
 Then /^the check for server warning 003 should not be displayed against the search after the (.*) conditional$/ do |format|
@@ -2038,10 +2102,10 @@ end
 
 Then /^the line number and line of code that triggered the warning(s)? should be displayed$/ do |multiple|
   if multiple.nil?
-    expect_line_shown 2, "log node['foo']"
+    expect_line_shown 1, "log node[:foo]"
   else
-    expect_line_shown 4, "    node['testing'] = 'bar'"
-    expect_line_shown 5, "    node['testing2'] = 'bar2'"
+    expect_line_shown 1, "node[:foo] = 'bar'"
+    expect_line_shown 2, "    node[:testing] = 'bar'"
   end
 end
 
@@ -2053,6 +2117,35 @@ Then /^the no leading cookbook name warning 029 should be (not )?shown$/ do |sho
   expect_warning('FC029', :line => 1, :expect_warning => should_not.nil?, :file => 'metadata.rb')
 end
 
+Then 'the node access warning 001 should be displayed for each match' do
+  expect_warning('FC001', :line => 1)
+  expect_warning('FC001', :line => 2)
+end
+
+Then 'the node access warning 001 should be displayed against the variables' do
+  expect_warning('FC001', :line => 4)
+  expect_warning('FC001', :line => 5)
+end
+
+Then 'the node access warning 001 should be displayed twice for the same line' do
+  expect_warning('FC001', :line => 1, :num_occurrences => 2)
+end
+
+Then 'the node access warning 001 should warn on lines 2 and 10 in that order' do
+  expected_warnings = [2, 10].map do |line|
+    "FC001: Use strings in preference to symbols to access node attributes: cookbooks/example/recipes/default.rb:#{line}"
+  end
+  expect_output(expected_warnings.join("\n"))
+end
+
+Then 'the node access warning 001 should be displayed for the recipe' do
+  expect_warning('FC001')
+end
+
+Then 'the node access warning 001 should not be displayed for the attributes' do
+  expect_warning("FC001", :file_type => :attributes, :line => 1, :expect_warning => false)
+end
+
 Then 'the prefer chef_gem to manual install warning 025 should be shown' do
   expect_warning('FC025', :line => nil)
 end
@@ -2115,7 +2208,7 @@ end
 
 Then 'the usage text should include an option for specifying tags that will fail the build' do
   expect_usage_option('f', 'epic-fail TAGS',
-    "Fail the build if any of the specified tags are matched ('any' -> fail on any match).")
+    "Fail the build based on tags. Use 'any' to fail on all warnings.")
 end
 
 Then /^the warnings shown should be (.*)$/ do |warnings|
