@@ -6,6 +6,11 @@ end
 require 'aruba/cucumber'
 require 'foodcritic'
 
+require 'minitest/autorun'
+require 'minitest/spec'
+
+MiniTest::Spec.new(nil)
+
 Before do
   @aruba_timeout_seconds = 300
 end
