@@ -1,10 +1,9 @@
 require 'nokogiri'
+require 'rufus-lru'
 
 module FoodCritic
-
   # Helper methods that form part of the Rules DSL.
   module Api
-
     include FoodCritic::AST
     include FoodCritic::XML
 
@@ -15,20 +14,20 @@ module FoodCritic
 
     # Find attribute access by type.
     def attribute_access(ast, options = {})
-      options = {:type => :any, :ignore_calls => false}.merge!(options)
+      options = { type: :any, ignore_calls: false }.merge!(options)
       return [] unless ast.respond_to?(:xpath)
 
       unless [:any, :string, :symbol, :vivified].include?(options[:type])
-        raise ArgumentError, "Node type not recognised"
+        fail ArgumentError, 'Node type not recognised'
       end
 
       case options[:type]
-        when :any then
+      when :any then
           vivified_attribute_access(ast, options) +
             standard_attribute_access(ast, options)
-        when :vivified then
+      when :vivified then
           vivified_attribute_access(ast, options)
-        else
+      else
           standard_attribute_access(ast, options)
       end
     end
@@ -38,8 +37,8 @@ module FoodCritic
       raise_unless_xpath!(ast)
       # TODO: This expression is too loose, but also will fail to match other
       # types of conditionals.
-      (! ast.xpath(%q{//*[self::if or self::unless]/*[self::aref or
-        child::aref or self::call]
+      (!ast.xpath(%q{//*[self::if or self::ifop or self::unless]/
+        *[self::aref or child::aref or self::call]
         [count(descendant::const[@value = 'Chef' or @value = 'Config']) = 2
           and
             (   count(descendant::ident[@value='solo']) > 0
@@ -47,13 +46,16 @@ module FoodCritic
             )
           ]}).empty?) ||
       ast.xpath('//if_mod[return][aref/descendant::ident/@value="solo"]/aref/
-        const_path_ref/descendant::const').map{|c|c['value']} == %w{Chef Config}
+        const_path_ref/descendant::const').map do |c|
+        c['value']
+      end == %w(Chef Config)
     end
 
-    # Is the [chef-solo-search library](https://github.com/edelight/chef-solo-search)
+    # Is the
+    # [chef-solo-search library](https://github.com/edelight/chef-solo-search)
     # available?
     def chef_solo_search_supported?(recipe_path)
-      return false if recipe_path.nil? || ! File.exists?(recipe_path)
+      return false if recipe_path.nil? || !File.exist?(recipe_path)
 
       # Look for the chef-solo-search library.
       #
@@ -61,21 +63,21 @@ module FoodCritic
       # is not under the same `cookbook_path` as the cookbook being checked.
       cbk_tree_path = Pathname.new(File.join(recipe_path, '../../..'))
       search_libs = Dir[File.join(cbk_tree_path.realpath,
-        '*/libraries/search.rb')]
+                                  '*/libraries/search.rb')]
 
       # True if any of the candidate library files match the signature:
       #
       #     class Chef
       #       def search
       search_libs.any? do |lib|
-        ! read_ast(lib).xpath(%q{//class[count(descendant::const[@value='Chef']
+        !read_ast(lib).xpath(%q{//class[count(descendant::const[@value='Chef']
           ) = 1]/descendant::def/ident[@value='search']}).empty?
       end
     end
 
     # The name of the cookbook containing the specified file.
     def cookbook_name(file)
-      raise ArgumentError, 'File cannot be nil or empty' if file.to_s.empty?
+      fail ArgumentError, 'File cannot be nil or empty' if file.to_s.empty?
 
       until (file.split(File::SEPARATOR) & standard_cookbook_subdirs).empty? do
         file = File.absolute_path(File.dirname(file.to_s))
@@ -86,9 +88,10 @@ module FoodCritic
       # We also need to consult the metadata in case the cookbook name has been
       # overridden there. This supports only string literals.
       md_path = File.join(file, 'metadata.rb')
-      if File.exists?(md_path)
+      if File.exist?(md_path)
         name = read_ast(md_path).xpath("//stmts_add/
-          command[ident/@value='name']/descendant::tstring_content/@value").to_s
+          command[ident/@value='name']/
+          descendant::tstring_content/@value").to_s
         return name unless name.empty?
       end
       File.basename(file)
@@ -109,14 +112,15 @@ module FoodCritic
       #     %w{foo bar baz}.each do |cbk|
       #       depends cbk
       #     end
-      deps = deps.to_a + word_list_values(ast, "//command[ident/@value='depends']")
-      deps.uniq.map{|dep| dep['value'].strip }
+      deps = deps.to_a +
+        word_list_values(ast, "//command[ident/@value='depends']")
+      deps.uniq.map { |dep| dep['value'].strip }
     end
 
     # The key / value pair in an environment or role ruby file
     def field(ast, field_name)
       if field_name.nil? || field_name.to_s.empty?
-        raise ArgumentError, "Field name cannot be nil or empty"
+        fail ArgumentError, 'Field name cannot be nil or empty'
       end
       ast.xpath("//command[ident/@value='#{field_name}']")
     end
@@ -125,14 +129,14 @@ module FoodCritic
     def field_value(ast, field_name)
       field(ast, field_name).xpath('args_add_block/descendant::tstring_content
         [count(ancestor::args_add) = 1][count(ancestor::string_add) = 1]
-        /@value').map{|a| a.to_s}.last
+        /@value').map { |a| a.to_s }.last
     end
 
     # Create a match for a specified file. Use this if the presence of the file
     # triggers the warning rather than content.
     def file_match(file)
-      raise ArgumentError, "Filename cannot be nil" if file.nil?
-      {:filename => file, :matched => file, :line => 1, :column => 1}
+      fail ArgumentError, 'Filename cannot be nil' if file.nil?
+      { filename: file, matched: file, line: 1, column: 1 }
     end
 
     # Find Chef resources of the specified type.
@@ -148,7 +152,7 @@ module FoodCritic
     #     find_resources(ast, :type => :service)
     #
     def find_resources(ast, options = {})
-      options = {:type => :any}.merge!(options)
+      options = { type: :any }.merge!(options)
       return [] unless ast.respond_to?(:xpath)
       scope_type = ''
       scope_type = "[@value='#{options[:type]}']" unless options[:type] == :any
@@ -170,7 +174,7 @@ module FoodCritic
     #     included_recipes(ast)
     #     included_recipes(ast, :with_partial_names => true)
     #
-    def included_recipes(ast, options = {:with_partial_names => true})
+    def included_recipes(ast, options = { with_partial_names: true })
       raise_unless_xpath!(ast)
 
       filter = ['[count(descendant::args_add) = 1]']
@@ -181,16 +185,18 @@ module FoodCritic
         filter << '[count(descendant::string_embexpr) = 0]'
       end
 
-      string_desc = '[descendant::args_add/string_literal]/descendant::tstring_content'
+      string_desc = '[descendant::args_add/string_literal]/
+        descendant::tstring_content'
       included = ast.xpath([
         "//command[ident/@value = 'include_recipe']",
-        "//fcall[ident/@value = 'include_recipe']/following-sibling::arg_paren",
+        "//fcall[ident/@value = 'include_recipe']/
+         following-sibling::arg_paren",
       ].map do |recipe_include|
         recipe_include + filter.join + string_desc
       end.join(' | '))
 
       # Hash keyed by recipe name with matched nodes.
-      included.inject(Hash.new([])){|h, i| h[i['value']] += [i]; h}
+      included.inject(Hash.new([])) { |h, i| h[i['value']] += [i]; h }
     end
 
     # Searches performed by the specified recipe that are literal strings.
@@ -206,28 +212,28 @@ module FoodCritic
       raise_unless_xpath!(node)
       pos = node.xpath('descendant::pos').first
       return nil if pos.nil?
-      {:matched => node.respond_to?(:name) ? node.name : '',
-       :line => pos['line'].to_i, :column => pos['column'].to_i}
+      { matched: node.respond_to?(:name) ? node.name : '',
+       line: pos['line'].to_i, column: pos['column'].to_i }
     end
 
     # Read the AST for the given Ruby source file
     def read_ast(file)
-      source = if file.to_s.split(File::SEPARATOR).include?('templates')
-        template_expressions_only(file)
+      @ast_cache ||= Rufus::Lru::Hash.new(5)
+      if @ast_cache.include?(file)
+        @ast_cache[file]
       else
-        File.read(file)
+        @ast_cache[file] = uncached_read_ast(file)
       end
-      build_xml(Ripper::SexpBuilder.new(source).parse)
     end
 
     # Retrieve a single-valued attribute from the specified resource.
     def resource_attribute(resource, name)
-      raise ArgumentError, "Attribute name cannot be empty" if name.empty?
+      fail ArgumentError, 'Attribute name cannot be empty' if name.empty?
       resource_attributes(resource)[name.to_s]
     end
 
     # Retrieve all attributes from the specified resource.
-    def resource_attributes(resource, options={})
+    def resource_attributes(resource, options = {})
       atts = {}
       name = resource_name(resource, options)
       atts[:name] = name unless name.empty?
@@ -239,8 +245,10 @@ module FoodCritic
     # Resources keyed by type, with an array of matching nodes for each.
     def resource_attributes_by_type(ast)
       result = {}
-      resources_by_type(ast).each do |type,resources|
-        result[type] = resources.map{|resource| resource_attributes(resource)}
+      resources_by_type(ast).each do |type, resources|
+        result[type] = resources.map do |resource|
+          resource_attributes(resource)
+        end
       end
       result
     end
@@ -248,13 +256,14 @@ module FoodCritic
     # Retrieve the name attribute associated with the specified resource.
     def resource_name(resource, options = {})
       raise_unless_xpath!(resource)
-      options = {:return_expressions => false}.merge(options)
+      options = { return_expressions: false }.merge(options)
       if options[:return_expressions]
         name = resource.xpath('command/args_add_block')
-        if name.xpath('descendant::string_add').size == 1 and
-          name.xpath('descendant::string_literal').size == 1 and
-          name.xpath('descendant::*[self::call or self::string_embexpr]').empty?
-            name.xpath('descendant::tstring_content/@value').to_s
+        if name.xpath('descendant::string_add').size == 1 &&
+          name.xpath('descendant::string_literal').size == 1 &&
+          name.xpath(
+            'descendant::*[self::call or self::string_embexpr]').empty?
+          name.xpath('descendant::tstring_content/@value').to_s
         else
           name
         end
@@ -267,7 +276,7 @@ module FoodCritic
     # Resources in an AST, keyed by type.
     def resources_by_type(ast)
       raise_unless_xpath!(ast)
-      result = Hash.new{|hash, key| hash[key] = Array.new}
+      result = Hash.new { |hash, key| hash[key] = Array.new }
       find_resources(ast).each do |resource|
         result[resource_type(resource)] << resource
       end
@@ -279,7 +288,7 @@ module FoodCritic
       raise_unless_xpath!(resource)
       type = resource.xpath('string(command/ident/@value)')
       if type.empty?
-        raise ArgumentError, "Provided AST node is not a resource"
+        fail ArgumentError, 'Provided AST node is not a resource'
       end
       type
     end
@@ -291,7 +300,7 @@ module FoodCritic
 
       checker = FoodCritic::ErrorChecker.new(str)
       checker.parse
-      ! checker.error?
+      !checker.error?
     end
 
     # Searches performed by the provided AST.
@@ -302,22 +311,24 @@ module FoodCritic
 
     # The list of standard cookbook sub-directories.
     def standard_cookbook_subdirs
-      %w{attributes definitions files libraries providers recipes resources
-         templates}
+      %w(attributes definitions files libraries providers recipes resources
+         templates)
     end
 
     # Platforms declared as supported in cookbook metadata
     def supported_platforms(ast)
       platforms = ast.xpath('//command[ident/@value="supports"]/
-        descendant::*[self::string_literal or self::symbol_literal][position() = 1]
+        descendant::*[self::string_literal or self::symbol_literal]
+        [position() = 1]
         [self::symbol_literal or count(descendant::string_add) = 1]/
         descendant::*[self::tstring_content | self::ident]')
-      platforms = platforms.to_a + word_list_values(ast, "//command[ident/@value='supports']")
+      platforms = platforms.to_a +
+        word_list_values(ast, "//command[ident/@value='supports']")
       platforms.map do |platform|
         versions = platform.xpath('ancestor::args_add[position() > 1]/
-	  string_literal/descendant::tstring_content/@value').map{|v| v.to_s}
-        {:platform => platform['value'], :versions => versions}
-      end.sort{|a,b| a[:platform] <=> b[:platform]}
+	  string_literal/descendant::tstring_content/@value').map { |v| v.to_s }
+        { platform: platform['value'], versions: versions }
+      end.sort { |a, b| a[:platform] <=> b[:platform] }
     end
 
     # Template filename
@@ -333,11 +344,11 @@ module FoodCritic
       end
     end
 
-    def templates_included(all_templates, template_path, depth=1)
-      raise RecursedTooFarError.new(template_path) if depth > 10
+    def templates_included(all_templates, template_path, depth = 1)
+      fail RecursedTooFarError.new(template_path) if depth > 10
       partials = read_ast(template_path).xpath('//*[self::command or
         child::fcall][descendant::ident/@value="render"]//args_add/
-        string_literal//tstring_content/@value').map{|p| p.to_s}
+        string_literal//tstring_content/@value').map { |p| p.to_s }
       Array(template_path) + partials.map do |included_partial|
         partial_path = Array(all_templates).find do |path|
           (Pathname.new(template_path).dirname + included_partial).to_s == path
@@ -367,21 +378,21 @@ module FoodCritic
       atts = {}
       resource.xpath("do_block/descendant::method_add_block[
         count(ancestor::do_block) = 1][brace_block | do_block]").each do |batt|
-          att_name = batt.xpath('string(method_add_arg/fcall/ident/@value)')
-          if att_name and ! att_name.empty? and batt.children.length > 1
-            atts[att_name] = batt.children[1]
-          end
+        att_name = batt.xpath('string(method_add_arg/fcall/ident/@value)')
+        if att_name && !att_name.empty? && batt.children.length > 1
+          atts[att_name] = batt.children[1]
+        end
       end
       atts
     end
 
     def block_depth(resource)
-      resource.path.split('/').group_by{|e|e}['method_add_block'].size
+      resource.path.split('/').group_by { |e|e }['method_add_block'].size
     end
 
     # Recurse the nested arrays provided by Ripper to create a tree we can more
     # easily apply expressions to.
-    def build_xml(node, doc = nil, xml_node=nil)
+    def build_xml(node, doc = nil, xml_node = nil)
       doc, xml_node = xml_document(doc, xml_node)
 
       if node.respond_to?(:each)
@@ -408,20 +419,21 @@ module FoodCritic
     end
 
     def extract_attribute_value(att, options = {})
-      if ! att.xpath('args_add_block[count(descendant::args_add)>1]').empty?
+      if !att.xpath('args_add_block[count(descendant::args_add)>1]').empty?
         att.xpath('args_add_block').first
-      elsif ! att.xpath('args_add_block/args_add/
+      elsif !att.xpath('args_add_block/args_add/
         var_ref/kw[@value="true" or @value="false"]').empty?
         att.xpath('string(args_add_block/args_add/
           var_ref/kw/@value)') == 'true'
-      elsif ! att.xpath('descendant::assoc_new').empty?
+      elsif !att.xpath('descendant::assoc_new').empty?
         att.xpath('descendant::assoc_new')
-      elsif ! att.xpath('descendant::int').empty?
+      elsif !att.xpath('descendant::int').empty?
         att.xpath('descendant::int/@value').to_s
       elsif att.xpath('descendant::symbol').empty?
-        if options[:return_expressions] and
-           (att.xpath('descendant::string_add').size != 1 or
-           ! att.xpath('descendant::*[self::call or self::string_embexpr]').empty?)
+        if options[:return_expressions] &&
+           (att.xpath('descendant::string_add').size != 1 ||
+            att.xpath('descendant::*[self::call or
+              self::string_embexpr]').any?)
           att
         else
           att.xpath('string(descendant::tstring_content/@value)')
@@ -438,33 +450,34 @@ module FoodCritic
     end
 
     def node_method?(meth, cookbook_dir)
-      chef_dsl_methods.include?(meth) || patched_node_method?(meth, cookbook_dir)
+      chef_dsl_methods.include?(meth) ||
+        patched_node_method?(meth, cookbook_dir)
     end
 
     def normal_attributes(resource, options = {})
       atts = {}
       resource.xpath('do_block/descendant::*[self::command or
         self::method_add_arg][count(ancestor::do_block) >= 1]').each do |att|
-          blocks = att.xpath('ancestor::method_add_block/method_add_arg/fcall')
-          next if blocks.any?{|a| block_depth(a) > block_depth(resource)}
-          att_name = att.xpath('string(ident/@value |
-            fcall/ident[@value="variables"]/@value)')
-          unless att_name.empty?
-            atts[att_name] = extract_attribute_value(att, options)
-          end
+        blocks = att.xpath('ancestor::method_add_block/method_add_arg/fcall')
+        next if blocks.any? { |a| block_depth(a) > block_depth(resource) }
+        att_name = att.xpath('string(ident/@value |
+          fcall/ident[@value="variables"]/@value)')
+        unless att_name.empty?
+          atts[att_name] = extract_attribute_value(att, options)
+        end
       end
       atts
     end
 
     def patched_node_method?(meth, cookbook_dir)
-      return false if cookbook_dir.nil? || ! Dir.exists?(cookbook_dir)
+      return false if cookbook_dir.nil? || !Dir.exists?(cookbook_dir)
 
       # TODO: Modify this to work with multiple cookbook paths
       cbk_tree_path = Pathname.new(File.join(cookbook_dir, '..'))
       libs = Dir[File.join(cbk_tree_path.realpath, '*/libraries/*.rb')]
 
       libs.any? do |lib|
-        ! read_ast(lib).xpath(%Q{//class[count(descendant::const[@value='Chef'])
+        !read_ast(lib).xpath(%Q{//class[count(descendant::const[@value='Chef'])
           > 0][count(descendant::const[@value='Node']) > 0]/descendant::def/
           ident[@value='#{meth.to_s}']}).empty?
       end
@@ -472,18 +485,28 @@ module FoodCritic
 
     def raise_unless_xpath!(ast)
       unless ast.respond_to?(:xpath)
-        raise ArgumentError, "AST must support #xpath"
+        fail ArgumentError, 'AST must support #xpath'
       end
     end
 
+    def uncached_read_ast(file)
+      source = if file.to_s.split(File::SEPARATOR).include?('templates')
+                 template_expressions_only(file)
+               else
+                 File.read(file)
+               end
+      build_xml(Ripper::SexpBuilder.new(source).parse)
+    end
+
     # XPath custom function
     class AttFilter
       def is_att_type(value)
         return [] unless value.respond_to?(:select)
         value.select do |n|
-          %w{
+          %w(
             automatic_attrs
             default
+            default!
             default_unless
             force_default
             force_override
@@ -491,10 +514,11 @@ module FoodCritic
             normal
             normal_unless
             override
+            override!
             override_unless
             set
             set_unless
-          }.include?(n.to_s)
+          ).include?(n.to_s)
         end
       end
     end
@@ -502,14 +526,14 @@ module FoodCritic
     def standard_attribute_access(ast, options)
       if options[:type] == :any
         [:string, :symbol].map do |type|
-          standard_attribute_access(ast, options.merge(:type => type))
+          standard_attribute_access(ast, options.merge(type: type))
         end.inject(:+)
       else
         type = if options[:type] == :string
-          'tstring_content'
-        else
-          '*[self::symbol or self::dyna_symbol]'
-        end
+                 'tstring_content'
+               else
+                 '*[self::symbol or self::dyna_symbol]'
+               end
         expr = '//*[self::aref_field or self::aref][count(method_add_arg) = 0]'
         expr += '[count(is_att_type(descendant::var_ref/ident/@value)) =
           count(descendant::var_ref/ident/@value)]'
@@ -527,24 +551,24 @@ module FoodCritic
 
     def template_expressions_only(file)
       exprs = Template::ExpressionExtractor.new.extract(File.read(file))
-      lines = Array.new(exprs.map{|e| e[:line]}.max || 0, '')
+      lines = Array.new(exprs.map { |e| e[:line] }.max || 0, '')
       exprs.each do |e|
-        lines[e[:line] -1] += ';' unless lines[e[:line] -1].empty?
-        lines[e[:line] -1] += e[:code]
+        lines[e[:line] - 1] += ';' unless lines[e[:line] - 1].empty?
+        lines[e[:line] - 1] += e[:code]
       end
       lines.join("\n")
     end
 
-    def vivified_attribute_access(ast, options={})
+    def vivified_attribute_access(ast, options = {})
       calls = ast.xpath(%Q{//*[self::call or self::field]
         [is_att_type(vcall/ident/@value) or is_att_type(var_ref/ident/@value)]
         #{ignore_attributes_xpath(options[:ignore])}
         [@value='.'][count(following-sibling::arg_paren) = 0]}, AttFilter.new)
       calls.select do |call|
-        call.xpath("aref/args_add_block").size == 0 and
-          (call.xpath("descendant::ident").size > 1 and
-            ! node_method?(call.xpath("ident/@value").to_s.to_sym,
-                options[:cookbook_dir]))
+        call.xpath('aref/args_add_block').size == 0 &&
+          (call.xpath('descendant::ident').size > 1 &&
+            !node_method?(call.xpath('ident/@value').to_s.to_sym,
+                          options[:cookbook_dir]))
       end.sort
     end
 
@@ -553,11 +577,10 @@ module FoodCritic
       if var_ref.empty?
         []
       else
-        ast.xpath(%Q{descendant::block_var/params/ident#{var_ref.first['value']}/
-          ancestor::method_add_block/call/descendant::tstring_content})
+        ast.xpath(%Q(descendant::block_var/params/
+          ident#{var_ref.first['value']}/ancestor::method_add_block/call/
+          descendant::tstring_content))
       end
     end
-
   end
-
 end
