@@ -16,24 +16,23 @@ module FoodCritic
       end
 
       def options
-        {:fail_tags => ['correctness'], # differs to default cmd-line behaviour
-         :cookbook_paths => @files,
-         :exclude_paths => ['test/**/*', 'spec/**/*', 'features/**/*']
+        {
+          fail_tags: ['correctness'], # differs to default cmd-line behaviour
+          cookbook_paths: @files,
+          exclude_paths: ['test/**/*', 'spec/**/*', 'features/**/*'],
+          context: false,
         }.merge(@options)
       end
 
       def define
-        desc "Lint Chef cookbooks"
+        desc 'Lint Chef cookbooks' unless ::Rake.application.last_comment
         task(name) do
           result = FoodCritic::Linter.new.check(options)
-          if result.warnings.any?
-            puts result
-          end
-
-          fail result.to_s if result.failed?
+          printer = options[:context] ? ContextOutput.new : SummaryOutput.new
+          printer.output(result) if result.warnings.any?
+          abort if result.failed?
         end
       end
-
     end
   end
 end
